from __future__ import annotations

import sys
from typing import Literal, TypeVar, Union

from django.contrib import admin
from django.contrib.admin.options import BaseModelAdmin

if sys.version_info >= (3, 11):
    from typing import Self, TypeAlias  # noqa: F401
else:
    from typing_extensions import Self, TypeAlias  # noqa: F401

AutoPopulate: TypeAlias = "bool | Literal['all', 'default', 'required']"

_K = TypeVar("_K")

# See https://github.com/typeddjango/django-stubs/blob/082955/django-stubs/utils/datastructures.pyi#L12-L14
_ListOrTuple: TypeAlias = Union[list[_K], tuple[_K, ...]]


# https://github.com/typeddjango/django-stubs/tree/master/django_stubs_ext
# For generic classes to work at runtime we need to define `__class_getitem__`.
# We're defining it here, instead of relying on django_stubs_ext, because
# we don't want every user setting up django_stubs_ext just for this feature.
def monkeypatch() -> None:
    classes = [
        admin.ModelAdmin,
        BaseModelAdmin,
    ]

    def class_getitem(cls: type, key: str | type | TypeVar):
        if isinstance(key, str) and hasattr(cls, key):
            # Fix django-cms compatibility:
            # https://github.com/django-cms/django-cms/issues/7948
            raise KeyError(f"Key '{key}' found as attribute, use getattr to access it.")
        return cls

    for cls in classes:
        cls.__class_getitem__ = classmethod(class_getitem)  # type: ignore[attr-defined]



from __future__ import annotations

from typing import TYPE_CHECKING

import django

if TYPE_CHECKING:
    from django.db.models.fields.reverse_related import ForeignObjectRel


def is_hidden(field: ForeignObjectRel) -> bool:
    return field.hidden


def clear_ForeignObjectRel_caches(field: ForeignObjectRel):
    """
    Django 5.1 Introduced caching for `accessor_name` props.

    We need to clear this cache when creating Translated field.

    https://github.com/django/django/commit/5e80390add100e0c7a1ac8e51739f94c5d706ea3#diff-e65b05ecbbe594164125af53550a43ef8a174f80811608012bc8e9e4ed575749
    """
    caches = ("accessor_name",)
    for name in caches:
        field.__dict__.pop(name, None)


if django.VERSION <= (5, 1):

    def is_hidden(field: ForeignObjectRel) -> bool:
        return field.is_hidden()



#!/usr/bin/env python
import sys

version = sys.argv[1]
if version.startswith("http"):
    print(version)
else:
    next_version = version[:-1] + "%d" % (int(version[-1]) + 1)
    print("Django>=%s,<%s" % (version, next_version))
